/**
 *
 */
package com.diagbot.facade;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.client.UserServiceClient;
import com.diagbot.dto.AddTagRetrievalDTO;
import com.diagbot.dto.GetRetrievalsByTagDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.RetrievalListDTO;
import com.diagbot.entity.Retrieval;
import com.diagbot.entity.RetrievalMapping;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.impl.RetrievalServiceImpl;
import com.diagbot.util.DateUtil;
import com.diagbot.util.UserUtils;
import com.diagbot.vo.AddTagRetrievalVO;
import com.diagbot.vo.DelRetrievalsByMapsVO;
import com.diagbot.vo.GetRetrievalsByTagVO;
import com.diagbot.vo.RetrievalListVO;

/**
 * @author rgb
 * @Description
 * @time 2018年12月5日下午4:53:59
 */
@Component
public class RetrievalFacade extends RetrievalServiceImpl {

    @Autowired
    private RetrievalMappingFacade retrievalMappingFacade;
    @Autowired
    private UserServiceClient userServiceClient;

    /**
     * 根据标签获取同义词
     *
     * @param getRetrievalsByTagVO
     * @return
     */
    public List<GetRetrievalsByTagDTO> getRetrievalsByTag(GetRetrievalsByTagVO getRetrievalsByTagVO) {

        return this.baseMapper.getRetrievalsByTag(getRetrievalsByTagVO);
    }

    /**
     * 根据映射关系删除标签的同义词
     *
     * @param delRetrievalsByMapsVO
     * @return
     */
    public Boolean delRetrievalsByMaps(DelRetrievalsByMapsVO delRetrievalsByMapsVO) {
    	QueryWrapper<RetrievalMapping> mappingQe = new QueryWrapper<>();
		String userId = UserUtils.getCurrentPrincipleID();
		RetrievalMapping retrievalMapping = new RetrievalMapping();
		retrievalMapping.setGmtModified(DateUtil.now());
		retrievalMapping.setModifier(userId);
		retrievalMapping.setIsDeleted("Y");
		mappingQe.in("id", delRetrievalsByMapsVO.getRetrievalMappingIds());
		retrievalMappingFacade.update(retrievalMapping, mappingQe);
		return true;
    }

    /**
     * 标签添加同义词
     *
     * @param addTagRetrievalVO
     * @return
     */
    public AddTagRetrievalDTO addTagRetrieval(AddTagRetrievalVO addTagRetrievalVO) {
        AddTagRetrievalDTO addTagRetrievalDTO = new AddTagRetrievalDTO();
        QueryWrapper<Retrieval> queryWrapper = new QueryWrapper<Retrieval>();
        queryWrapper.eq("name", addTagRetrievalVO.getRetrievalName());
        queryWrapper.eq("is_deleted", "N");
        Retrieval retrieval = this.getOne(queryWrapper);
        String userId = UserUtils.getCurrentPrincipleID();
        if (retrieval == null) {
            retrieval = new Retrieval();
            retrieval.setName(addTagRetrievalVO.getRetrievalName());
            retrieval.setCreator(userId);
            retrieval.setGmtCreate(DateUtil.now());
        } else {
            QueryWrapper<RetrievalMapping> queryWrapper2 = new QueryWrapper<RetrievalMapping>();
            queryWrapper2.eq("retrieval_id", retrieval.getId());
            queryWrapper2.eq("question_id", addTagRetrievalVO.getQuestionId());
            retrievalMappingFacade.remove(queryWrapper2);
        }

        retrieval.setSpell(addTagRetrievalVO.getRetrievalSpell());
        this.saveOrUpdate(retrieval);

        RetrievalMapping retrievalMapping = new RetrievalMapping();
        retrievalMapping.setQuestionId(addTagRetrievalVO.getQuestionId());
        retrievalMapping.setRetrievalId(retrieval.getId());
        retrievalMapping.setShowType(addTagRetrievalVO.getRetrievalType());
        retrievalMapping.setCreator(userId);
        retrievalMapping.setGmtCreate(DateUtil.now());
        retrievalMappingFacade.save(retrievalMapping);

        addTagRetrievalDTO.setRetrievalId(retrieval.getId());
        addTagRetrievalDTO.setRetrievalMappingId(retrievalMapping.getId());
        return addTagRetrievalDTO;
    }
    
    /**
	 * 获取标签同义词列表
	 * @param retrievalListVO
	 * @return
	 */
    public IPage<RetrievalListDTO> retrievalList(RetrievalListVO retrievalListVO) {
        IPage<RetrievalListDTO> ipage = this.getRetrievalList(retrievalListVO);
        List<String> ids = ipage.getRecords().stream().map(i->i.getOperatorName()).collect(Collectors.toList());
        RespDTO<Map<String, String>> respDTO = userServiceClient.getUserInfoByIds(ids);
        if (respDTO == null || !"0".equals(respDTO.code)) {
            throw new CommonException(CommonErrorCode.RPC_ERROR,
                    "获取用户信息失败");
        }
        ipage.getRecords().forEach(i->i.setOperatorName(respDTO.data.get(i.getOperatorName())));
        return ipage;
    }

}
